package ru.vyatsu.qr_access_api.booking.service

import com.fasterxml.jackson.databind.node.JsonNodeFactory
import com.fasterxml.jackson.databind.node.ObjectNode
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import ru.vyatsu.qr_access_api.booking.repository.BookingRepository
import ru.vyatsu.qr_access_api.booking.request.BookRequest
import ru.vyatsu.qr_access_api.common.exception.ValidationException
import ru.vyatsu.qr_access_api.email.repository.EmailRepository

@Service
class BookingService(val bookingRepository: BookingRepository, val emailRepository: EmailRepository) {
    fun book(request: BookRequest): String {
        // TODO: Произвести еще валидации, если нужны
        val date = request.startDateTime.toLocalDate()
        if (date != request.endDateTime.toLocalDate()) {
            throw ValidationException(
                "startDateTime, endDateTime",
                "startDateTime and endDateTime have to have the same day"
            )
        }
        val startTime = request.startDateTime.toLocalTime()
        val endTime = request.endDateTime.toLocalTime()

        return bookingRepository.book(request.doorId, date, startTime, endTime, request.clientEmail)
    }

    @Transactional
    fun bookPayed(bookId: String) {
        bookingRepository.markBookingPayed(bookId)
        val (qrCode, email) = bookingRepository.findRentWithClientById(bookId)
        val additionalData = JsonNodeFactory.instance.objectNode()
        additionalData.set<ObjectNode>("qr_code", JsonNodeFactory.instance.textNode(qrCode))
        emailRepository.createEmailOutboxRecord(email, "qr_code", additionalData)
    }
}