package ru.vyatsu.qr_access_api.config

import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpMethod
import org.springframework.security.config.Customizer
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.web.SecurityFilterChain
import org.springframework.web.cors.CorsConfiguration

@Configuration
@EnableWebSecurity(debug = true)
class SecurityConfig {

    @Bean
    fun defaultSecurityFilterChain(http: HttpSecurity): SecurityFilterChain {
        return http.authorizeHttpRequests {
            it.requestMatchers("/public/**", "/error").permitAll()
                .anyRequest().authenticated()
        }
            .oauth2ResourceServer { it.jwt(Customizer.withDefaults()) }
            .cors { c ->
                c.configurationSource {
                    val config = CorsConfiguration()
                    config.addAllowedOrigin("http://localhost:3000")
                    config.allowedMethods = listOf(
                        HttpMethod.GET.name(),
                        HttpMethod.POST.name()
                    )
                    config.allowedHeaders = listOf(
                        HttpHeaders.CONTENT_TYPE
                    )
                    config
                }
            }
            .csrf { c -> c.disable() }
            .build()
    }
}