package ru.vyatsu.qr_access_api.email.job

import io.nayuki.qrcodegen.QrCode
import org.springframework.mail.javamail.JavaMailSender
import org.springframework.mail.javamail.MimeMessageHelper
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Service
import ru.vyatsu.qr_access_api.common.logger
import ru.vyatsu.qr_access_api.email.repository.EmailRepository
import java.awt.image.BufferedImage
import java.io.ByteArrayInputStream
import java.io.ByteArrayOutputStream
import java.util.*
import javax.imageio.ImageIO


@Service
class EmailOutboxSendJob(val repository: EmailRepository, val mailSender: JavaMailSender) {

    @Scheduled(cron = "*/5 * * * * *")
    fun sendEmails() {
        val messagesToSend = repository.findRecordsToSendWithBlock()

        messagesToSend.forEach {
            val msg = mailSender.createMimeMessage()

            val helper = MimeMessageHelper(msg, true)

            helper.setFrom("kashiuno@yandex.ru")
            helper.setTo(it.email)
            helper.setSubject("Приобретение qr-кода")
            helper.setText("Вы приобрели проход в коворкинг на нашем сайте. qr-код во вложении. QR-код нужно приложить к сканеру соответствующей двери и она откроется")

            val qrCode = it.additionalData.get("qr_code")
            val code = QrCode.encodeText(qrCode.asText(), QrCode.Ecc.MEDIUM)
            val image = toImage(code, 3, 2)
            val os = ByteArrayOutputStream()
            ImageIO.write(image, "png", os)
            val imageIS = ByteArrayInputStream(os.toByteArray())
            helper.addAttachment("qr_code.png") { imageIS }
        }

        val recordsWasDeletedCount = repository.deleteRecordsToSend(messagesToSend.map { it.id })
        logger().info("Emails sent {}", recordsWasDeletedCount)
    }

    companion object {
        private fun toImage(qr: QrCode, scale: Int, border: Int, lightColor: Int, darkColor: Int): BufferedImage {
            Objects.requireNonNull(qr)
            require(!(scale <= 0 || border < 0)) { "Value out of range" }
            require(!(border > Int.MAX_VALUE / 2 || qr.size + border * 2L > Int.MAX_VALUE / scale)) { "Scale or border too large" }

            val result = BufferedImage(
                (qr.size + border * 2) * scale,
                (qr.size + border * 2) * scale,
                BufferedImage.TYPE_INT_RGB
            )
            for (y in 0 until result.height) {
                for (x in 0 until result.width) {
                    val color = qr.getModule(x / scale - border, y / scale - border)
                    result.setRGB(x, y, if (color) darkColor else lightColor)
                }
            }
            return result
        }

        fun toImage(qr: QrCode?, scale: Int, border: Int): BufferedImage {
            return toImage(qr!!, scale, border, 0xFFFFFF, 0x000000)
        }
    }
}