package ru.vyatsu.qr_access_api.email.repository

import com.fasterxml.jackson.databind.JsonNode
import com.fasterxml.jackson.databind.ObjectMapper
import org.springframework.jdbc.core.BatchPreparedStatementSetter
import org.springframework.jdbc.core.JdbcTemplate
import org.springframework.stereotype.Repository
import ru.vyatsu.qr_access_api.email.repository.entity.EmailOutbox
import java.sql.PreparedStatement
import java.util.*

private const val INSERT_EMAIL_OUTBOX_RECORD =
    "insert into email_outbox(id, email, template, additional_info) values (?, ?, ?, (to_json(?::json)))"

@Repository
class EmailRepository(private val template: JdbcTemplate, private val om: ObjectMapper) {
    fun createEmailOutboxRecord(email: String, msgTemplate: String, additionalInfo: JsonNode) {
        val insertedCount = template.update {
            val stmt = it.prepareStatement(INSERT_EMAIL_OUTBOX_RECORD)
            stmt.setString(1, UUID.randomUUID().toString())
            stmt.setString(2, email)
            stmt.setString(3, msgTemplate)

            stmt.setObject(4, om.writeValueAsString(additionalInfo))
            stmt
        }

        if (insertedCount != 1)
            throw RuntimeException("Inserted rows should be equals to 1")
    }

    fun findRecordsToSendWithBlock(): List<EmailOutbox> {
        return template.query("select id, email, template, additional_info from email_outbox for update", { rs, _ ->
            EmailOutbox(
                rs.getString("id"),
                rs.getString("email"),
                rs.getString("template"),
                om.readTree(rs.getString("additional_info"))
            )
        })
    }

    fun deleteRecordsToSend(ids: List<String>): Int {
        val query = "delete from email_outbox where id = ?"
        return template.batchUpdate(query, object : BatchPreparedStatementSetter {
            override fun setValues(ps: PreparedStatement, i: Int) {
                ps.setString(1, ids[i])
            }

            override fun getBatchSize(): Int = ids.size
        })
            .sum()
    }
}