package ru.vyatsu.qr_access_api.qr.repository

import org.springframework.jdbc.core.JdbcTemplate
import org.springframework.stereotype.Repository
import ru.vyatsu.models.QrCode
import java.time.LocalDateTime
import java.time.ZoneId
import java.util.*

private const val GET_ACTUAL_QRS_BY_UNIT_ID = """
            SELECT r.start_time, r.end_time, r.date, r.door_id, r.qr_code FROM rents r
            JOIN doors d ON (d.id = r.door_id)
            JOIN oauth2_registered_client c ON (c.client_id = d.unit_id)
            WHERE c.client_id = ? AND r.start_time <= CURRENT_TIMESTAMP AND r.end_time >= CURRENT_TIMESTAMP
            """

@Repository
class QrRepository(private val template: JdbcTemplate) {

    fun getActualQrCodesByUnitId(unitId: String): List<QrCode> {
        val query = GET_ACTUAL_QRS_BY_UNIT_ID.trimIndent()
        return template.query({ conn ->
            val stmt = conn.prepareStatement(query)
            stmt.setString(1, unitId)
            stmt
        }, { rs, _ ->
            QrCode(
                LocalDateTime.of(rs.getDate("date").toLocalDate(), rs.getTime("start_time").toLocalTime()).atZone(ZoneId.systemDefault()).toOffsetDateTime(),
                LocalDateTime.of(rs.getDate("date").toLocalDate(), rs.getTime("end_time").toLocalTime()).atZone(ZoneId.systemDefault()).toOffsetDateTime(),
                UUID.fromString(rs.getString("door_id")),
                UUID.fromString(rs.getString("qr_code"))
            )
        })
    }

}