package ru.vyatsu.qr_access_api.slots.service

import org.springframework.stereotype.Service
import ru.vyatsu.qr_access_api.slots.repository.SlotRepository
import ru.vyatsu.qr_access_api.slots.response.Door
import ru.vyatsu.qr_access_api.slots.response.Slot
import ru.vyatsu.qr_access_api.slots.response.SlotResponse
import ru.vyatsu.qr_access_api.slots.response.SlotStatus
import java.time.LocalDate
import java.time.LocalTime

@Service
class SlotService(private val repository: SlotRepository) {
    fun getAllSlotsByPartner(partnerId: String): SlotResponse {
        // TODO: Учитывать количество мест и переписать логику
        val doorsWithSchedule = repository.findAllDoorsWithScheduleByPartnerId(partnerId)
        val doorRents = repository.findRentDateTimesByPartnerId(partnerId)
        val doors: MutableList<Door> = mutableListOf()
        doorsWithSchedule.forEach { d ->
            val slots: MutableList<Slot> = mutableListOf()
            d.schedule.forEach { sch ->
                val rentTimes = doorRents.dates[d.id]?.rentDate?.get(sch.date) ?: listOf()
                createIntermediateSlot(LocalTime.MIN, sch.startTime, sch.date, SlotStatus.OUT_OF_WORKING_TIME)
                    ?.also { slots.add(it) }
                rentTimes.forEach { rt ->
                    val lastSlot: Slot? = if (slots.lastIndex == -1) null else slots.last()
                    createIntermediateSlot(lastSlot?.endTime, rt.startTime, sch.date, SlotStatus.FREE)
                        ?.also { slots.add(it) }
                    slots.add(Slot(rt.startTime, rt.endTime, sch.date, SlotStatus.BOOKED))
                }
                var lastSlot: Slot? = if (slots.lastIndex == -1) null else slots.last()
                createIntermediateSlot(lastSlot?.endTime, sch.endTime, sch.date, SlotStatus.FREE)
                    ?.also { slots.add(it) }
                lastSlot = if (slots.lastIndex == -1) null else slots.last()
                createIntermediateSlot(lastSlot?.endTime, LocalTime.MAX, sch.date, SlotStatus.OUT_OF_WORKING_TIME)
                    ?.also { slots.add(it) }
            }
            doors.add(Door(d.id, d.description, slots))
        }
        return SlotResponse(doors)
    }

    private fun createIntermediateSlot(
        startTime: LocalTime?,
        endTime: LocalTime,
        date: LocalDate,
        status: SlotStatus
    ): Slot? {
        return if (startTime != null && startTime != endTime) Slot(startTime, endTime, date, status) else null
    }
}