package ru.vyatsu.qr_access_api.database.utils

import org.springframework.jdbc.core.JdbcTemplate
import java.time.LocalDateTime
import java.time.OffsetDateTime

private const val INSERT_CLIENT_QUERY =
    """INSERT INTO oauth2_registered_client(id, client_id, client_id_issued_at, client_secret, client_secret_expires_at, client_name, client_authentication_methods, authorization_grant_types, redirect_uris, post_logout_redirect_uris, scopes, client_settings, token_settings) 
       VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, to_json(?::json), to_json(?::json))"""

class InsertDatabaseHelper(private val template: JdbcTemplate) {
    fun insertQr(
        keyCode: String,
        doorId: String,
        startDateTime: OffsetDateTime,
        endDateTime: OffsetDateTime
    ): Int {
        return template.update("INSERT INTO qrs(key_code, door_id, start_date_time, end_date_time) VALUES (?, ?, ?, ?)") { ps ->
            ps.setString(1, keyCode)
            ps.setString(2, doorId)
            ps.setObject(3, startDateTime)
            ps.setObject(4, endDateTime)
        }
    }

    fun insertDoor(id: String, unitId: String): Int {
        return template.update("INSERT INTO doors(id, unit_id) VALUES (?, ?)") { ps ->
            ps.setString(1, id)
            ps.setString(2, unitId)
        }
    }

    fun insertClient(id: String): Int {
        return template.update(INSERT_CLIENT_QUERY) { ps ->
            ps.setString(1, id)
            ps.setString(2, id)
            ps.setObject(3, LocalDateTime.now())
            ps.setString(4, "secret")
            ps.setObject(5, LocalDateTime.now())
            ps.setString(6, id)
            ps.setString(7, "client_secret_post")
            ps.setString(8, "client_credentials")
            ps.setString(9, "http://localhost:8080")
            ps.setString(10, "http://localhost:8080")
            ps.setString(11, "")
            ps.setString(12, "{}")
            ps.setString(13, "{}")
        }
    }
}